!!
!! This module computes diffusion coefficients for continental points.
!!
!! @author Marie-Alice Foujols and Jan Polcher
!! @Version : $Revision: 1.22.2.1 $, $Date: 2003/12/04 13:21:02 $
!! 
!! $Header: /home/ssipsl/CVSREP/ORCHIDEE/src_sechiba/diffuco.f90,v 1.22.2.1 2003/12/04 13:21:02 ssipsl Exp $
!! 
MODULE diffuco

  ! modules used :
  USE constantes
  USE constantes_veg
  USE sechiba_io
  USE ioipsl
  USE constantes_co2

  IMPLICIT NONE

  ! public routines :
  ! diffuco_main only
  PRIVATE
  PUBLIC :: diffuco_main,diffuco_clear

  !
  ! variables used inside diffuco module : declaration and initialisation
  !
  INTEGER(short), PARAMETER                         :: nlai = 20
  LOGICAL, SAVE                                     :: l_first_diffuco = .TRUE.  !! Initialisation has to be done one time
  CHARACTER*80                                      :: var_name                  !! To store variables names for I/O
  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:,:,:)  :: leaf_ci                   !! intercellular CO2 concentration (ppm)
  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: rstruct                   !! architectural resistance
  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:)      :: raero                     !! Aerodynamic resistance
  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:)      :: qsatt                     !! Surface saturated humidity

CONTAINS

  !! 
  !! Main routine for *diffuco* module.
  !! - called only one time for initialisation
  !! - called every time step
  !! - called one more time at last time step for writing _restart_ file
  !!
  !! Algorithm:
  !! - call diffuco_aero for aerodynamic transfer coeficient
  !! - call diffuco_snow for partial beta coefficient : sublimation
  !! - call diffuco_inter for partial beta coefficient : interception for each type of vegetation
  !! - call diffuco_bare for partial beta coefficient : bare soil
  !! - call diffuco_trans for partial beta coefficient : transpiration for each type of vegetation
  !! - call diffuco_comb for alpha and beta coefficient
  !!
  !! @call diffuco_aero
  !! @call diffuco_snow
  !! @call diffuco_inter
  !! @call diffuco_bare
  !! @call diffuco_trans
  !! @call diffuco_comb
  !!
  SUBROUTINE diffuco_main (kjit, kjpindex, dtradia, ldrestart_read, ldrestart_write, index, indexveg, u, v, &
     & zlev, z0, roughheight, temp_sol, temp_air, rau, q_cdrag, qsurf, qair, pb, &
     & hdry, evapot, snow, frac_nobio, snow_nobio, totfrac_nobio, &
     & swnet, swdown, ccanopy, humrel, veget, veget_max, lai, qsintveg, qsintmax, assim_param, &
     & vbeta , valpha, vbeta1, vbeta2, vbeta3, vbeta4, vbetaco2, rveget, cimean, rsol, rest_id, hist_id)  

    ! interface description:
    ! input scalar 
    INTEGER(long), INTENT(in)                         :: kjit             !! Time step number 
    INTEGER(long), INTENT(in)                         :: kjpindex         !! Domain size
    INTEGER(long),INTENT (in)                         :: rest_id          !! _Restart_ file identifier
    INTEGER(long),INTENT (in)                         :: hist_id          !! _History_ file identifier
    REAL(stnd), INTENT (in)                           :: dtradia          !! Time step in seconds
    LOGICAL, INTENT(in)                               :: ldrestart_read   !! Logical for restart file to read
    LOGICAL, INTENT(in)                               :: ldrestart_write  !! Logical for restart file to write
    ! input fields
    INTEGER(long),DIMENSION (kjpindex), INTENT (in)     :: index            !! Indeces of the points on the map
    INTEGER(long),DIMENSION (kjpindex*nvm), INTENT (in) :: indexveg         !! Indeces of the points on the 3D map
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: u                !! Lowest level wind speed 
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: v                !! Lowest level wind speed
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: zlev             !! Height of first layer
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: z0               !! Surface roughness
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: roughheight      !! Effective height for roughness
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: temp_sol         !! Skin temperature in Kelvin
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: temp_air         !! Lowest level temperature in Kelvin
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: rau              !! Density
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: qsurf            !! near surface specific humidity
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: qair             !! Lowest level specific humidity
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: snow             !! Snow mass
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: pb               !! Surface level pressure
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: hdry             !! Dry soil heigth in meters
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: evapot           !! Soil Potential Evaporation
    REAL(stnd),DIMENSION (kjpindex,nnobio), INTENT (in) :: frac_nobio     !! Fraction of ice,lakes,cities,...
    REAL(stnd),DIMENSION (kjpindex,nnobio), INTENT (in) :: snow_nobio     !! Snow on ice,lakes,cities,...
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: totfrac_nobio    !! Total fraction of ice+lakes+cities+...
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: swnet            !! Net surface short-wave flux
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: swdown           !! Down-welling surface short-wave flux
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: ccanopy          !! CO2 concentration inside the canopy
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)  :: veget            !! Fraction of vegetation type 
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)  :: veget_max        !! Max. fraction of vegetation type (LAI->infty)
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)  :: lai              !! Leaf area index
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)  :: qsintveg         !! Water on vegetation due to interception
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)  :: qsintmax         !! Maximum water on vegetation for interception
    REAL(stnd),DIMENSION (kjpindex,nvm,npco2), INTENT (in):: assim_param  !! min+max+opt temps, vcmax, vjmax for photosynthesis
    ! modified fields 
    REAL(stnd),DIMENSION (kjpindex, nvm), INTENT (inout) :: humrel        !! Moisture stress
    REAL(stnd),DIMENSION (kjpindex), INTENT (inout)   :: q_cdrag          !! Surface drag
    ! output fields
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)     :: vbeta            !! Total beta coefficient
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)     :: valpha           !! Total alpha coefficient
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)     :: vbeta1           !! Beta for sublimation
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)     :: vbeta4           !! Beta for bare soil evaporation
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (out) :: vbetaco2         !! STOMATE: Beta for CO2
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)     :: rsol             !! Bare soil evaporation resistance
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (out) :: vbeta2           !! Beta for interception loss
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (out) :: vbeta3           !! Beta for transpiration
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (out) :: rveget           !! Surface resistance for the vegetatuon
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (out) :: cimean           !! STOMATE: mean intercellular ci (see enerbil)
    ! Local
    LOGICAL                                           :: do_aero          !! True if do_aero needs to be called
    INTEGER(short)                                    :: ilai
    CHARACTER*4                                       :: laistring

    ! do initialisation if needed

    IF (l_first_diffuco) THEN

        IF (long_print) WRITE (numout,*) ' call diffuco_init '

        CALL diffuco_init(kjit, ldrestart_read, kjpindex, index, rest_id)

        RETURN

    ENDIF

    !
    ! prepares restart file for the next simulation
    !
    IF (ldrestart_write) THEN

        IF (long_print) WRITE (numout,*) ' we have to complete restart file with DIFFUCO variables '

        var_name= 'rstruct'
        CALL restput (rest_id, var_name, kjpindex, nvm, 1, kjit, rstruct, 'scatter',  kjpindex, index)
  
        ! the following variable is written only if CO2 was calculated
        IF ( control%ok_co2 ) THEN

          DO ilai = 1, nlai
  
            ! variable name is somewhat complicated as ioipsl does not allow 3d variables for the moment...
            write(laistring,'(i4)') ilai
            laistring=ADJUSTL(laistring)
            var_name='leaf_ci_'//laistring(1:LEN_TRIM(laistring))
  
            CALL restput (rest_id, var_name, kjpindex, nvm, 1, kjit, leaf_ci(:,:,ilai), 'scatter',  kjpindex, index)
  
          ENDDO

        ENDIF
  
      RETURN
  
    END IF

    !
    ! calculs des differents coefficients
    !
    do_aero = .FALSE.
    IF ( ABS(MAXVAL(q_cdrag)) .LE. EPSILON(q_cdrag)) do_aero = .TRUE.

    IF ( do_aero ) THEN
       CALL diffuco_aero (kjpindex, kjit, u, v, zlev, z0, roughheight, veget_max, temp_sol, temp_air, &
            &             qsurf, qair, q_cdrag)
    ENDIF
    CALL diffuco_raerod (kjpindex, u, v, q_cdrag, raero)

    !
    ! An estimation of the satturated humidity at the surface
    !

    CALL qsatcalc (kjpindex, temp_sol, pb, qsatt)

    !
    ! beta coefficient for sublimation
    !

    CALL diffuco_snow (kjpindex, dtradia, qair, qsatt, rau, u, v, q_cdrag, &
         & snow, frac_nobio, totfrac_nobio, snow_nobio, vbeta1)

    !
    ! beta coefficient for interception
    !

    CALL diffuco_inter (kjpindex, dtradia, qair, qsatt, rau, u, v, q_cdrag, veget, &
       & qsintveg, qsintmax, rstruct, vbeta2) 

    !
    ! beta coefficient for bare soil
    !

    CALL diffuco_bare (kjpindex, dtradia, u, v, q_cdrag, hdry, evapot, humrel, veget, vbeta4, rsol) 

    !
    ! beta coefficient for transpiration
    !

    IF ( control%ok_co2 ) THEN

      CALL diffuco_trans_co2 (kjpindex, dtradia, swdown, temp_air, pb, qair, rau, u, v, q_cdrag, humrel, &
                              assim_param, ccanopy, &
                              veget, veget_max, lai, qsintveg, qsintmax, vbeta3, rveget, rstruct, cimean, vbetaco2)

    ELSE

      CALL diffuco_trans (kjpindex, dtradia, swnet, temp_air, pb, qair, rau, u, v, q_cdrag, humrel, &
                          veget, veget_max, lai, qsintveg, qsintmax, vbeta3, rveget, rstruct, cimean, vbetaco2)

    ENDIF

    !
    ! combination of coefficient : alpha and beta coefficient
    !

    CALL diffuco_comb (kjpindex, dtradia, humrel, rau, u, v, q_cdrag, pb, qair, temp_sol, temp_air, snow, &
       & veget, vbeta1, vbeta2, vbeta3, vbeta4, valpha, vbeta)    

    IF ( .NOT. almaoutput ) THEN
       CALL histwrite(hist_id, 'rstruct', kjit, rstruct, kjpindex*nvm, indexveg)
       CALL histwrite(hist_id, 'raero', kjit, raero, kjpindex, indexveg)
    ELSE

    ENDIF
    !
    !
    IF (long_print) WRITE (numout,*) ' diffuco_main done '

  END SUBROUTINE diffuco_main

  !! Algorithm:
  !! - dynamic allocation for local array 
  !!
  SUBROUTINE diffuco_init(kjit, ldrestart_read, kjpindex, index, rest_id)

    ! interface description
    ! input scalar 
    INTEGER(long), INTENT (in)                     :: kjit               !! Time step number 
    LOGICAL,INTENT (in)                            :: ldrestart_read     !! Logical for restart file to read
    INTEGER(long), INTENT (in)                     :: kjpindex           !! Domain size
    INTEGER(long),DIMENSION (kjpindex), INTENT (in):: index              !! Indeces of the points on the map
    INTEGER(long), INTENT (in)                     :: rest_id            !! _Restart_ file identifier 
    ! input fields
    ! output scalar
    ! output fields

    ! local declaration
    INTEGER(short)                  :: ier, jv
    INTEGER(short)                  :: ilai
    CHARACTER*4                     :: laistring

    !
    ! initialisation
    !
    IF (l_first_diffuco) THEN 
        l_first_diffuco=.FALSE.
    ELSE 
        WRITE (numout,*) ' l_first_diffuco false . we stop '
        STOP 'diffuco_init'
    ENDIF

    ! allocate only if CO2 is calculated
    IF ( control%ok_co2 ) THEN

      ALLOCATE (leaf_ci(kjpindex,nvm,nlai),stat=ier)
      IF (ier.NE.0) THEN
          WRITE (numout,*) ' error in leaf_ci allocation. We stop. We need kjpindex*nvm*nlai words = ',&
            kjpindex*nvm*nlai
          STOP 'diffuco_init'
      END IF

    ENDIF

    ALLOCATE (rstruct(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in rstuct allocation. We stop. We need kjpindex x nvm words = ' ,kjpindex,' x ' ,nvm,&
           & ' = ',kjpindex*nvm
        STOP 'diffuco_init'
    END IF
    ALLOCATE (raero(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in raero allocation. We stop. We need kjpindex x nvm words = ', kjpindex
        STOP 'diffuco_init'
    END IF

    ALLOCATE (qsatt(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in qsatt allocation. We stop. We need kjpindex x nvm words = ', kjpindex
        STOP 'diffuco_init'
    END IF

    IF (ldrestart_read) THEN

        IF (long_print) WRITE (numout,*) ' we have to read a restart file for DIFFUCO variables'

        var_name='rstruct' ;
        CALL ioconf_setatt('UNITS', 's/m')
        CALL ioconf_setatt('LONG_NAME','Structural resistance')
        CALL restget (rest_id, var_name, kjpindex, nvm, 1, kjit, .TRUE., rstruct, "gather", kjpindex, index)
        !
        IF ( MINVAL(rstruct) .EQ. MAXVAL(rstruct) .AND.  MAXVAL(rstruct) .EQ. val_exp ) THEN
        !
           DO jv = 1, nvm
              rstruct(:,jv) = rstruct_const(jv)
           ENDDO

        ENDIF

        ! the following variable is read only if CO2 is calculated
        IF ( control%ok_co2 ) THEN

           CALL ioconf_setatt('UNITS', 'ppm')
           CALL ioconf_setatt('LONG_NAME','Leaf CO2')

           DO ilai = 1, nlai

              ! variable name is somewhat complicated as ioipsl does not allow 3d variables for the moment...
              write(laistring,'(i4)') ilai
              laistring=ADJUSTL(laistring)
              var_name='leaf_ci_'//laistring(1:LEN_TRIM(laistring))
              
              CALL restget (rest_id, var_name, kjpindex, nvm, 1, kjit, .TRUE., leaf_ci(:,:,ilai), "gather", kjpindex, index)
              
           ENDDO
           !
           !Config Key  = DIFFUCO_LEAFCI
           !Config Desc = Initial leaf CO2 level if not found in restart
           !Config Def  = 233.
           !Config Help = The initial value of leaf_ci if its value is not found
           !Config        in the restart file. This should only be used if the model is
           !Config        started without a restart file.
           !
           CALL setvar (leaf_ci, val_exp,'DIFFUCO_LEAFCI', 233._stnd)
           
        ENDIF
        
    ENDIF

      CALL getin('RSOL_CSTE',rsol_cste)
      CALL getin('RVEG_CSTE',rveg_cste)

      print*,'DANS DIFFUCO_INIT , RSOL_CSTE,RVEG_CSTE=',rsol_cste,rveg_cste

    IF (long_print) WRITE (numout,*) ' diffuco_init done '

  END SUBROUTINE diffuco_init
  SUBROUTINE diffuco_clear()

         l_first_diffuco=.TRUE.

      IF (ALLOCATED (leaf_ci)) DEALLOCATE (leaf_ci)
      IF (ALLOCATED (rstruct)) DEALLOCATE (rstruct)
      IF (ALLOCATED (raero)) DEALLOCATE (raero)
 
  END SUBROUTINE diffuco_clear

  !! This routine computes aerothermic coeFficient if required
  !! see logical *ldq_cdrag_from_gcm*
  !!
  SUBROUTINE diffuco_aero (kjpindex, kjit, u, v, zlev, z0, roughheight, veget_max, temp_sol, temp_air, &
       &                   qsurf, qair, q_cdrag)

    ! interface description
    ! input scalar 
    INTEGER(long), INTENT(in)                               :: kjpindex, kjit         !! Domain size
    ! input fields
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: u                !! Lowest level wind speed 
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: v                !! Lowest level wind speed
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: zlev             !! Height of first layer
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: z0               !! Surface roughness
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: roughheight      !! Effective roughness height
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)        :: veget_max        !! Fraction of vegetation type
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: temp_sol         !! Skin temperature in Kelvin
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: temp_air         !! Lowest level temperature in Kelvin
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: qsurf            !! near surface specific humidity
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: qair             !! Lowest level specific humidity

    ! output scalar
    ! output fields
    REAL(stnd),DIMENSION (kjpindex), INTENT (inout)         :: q_cdrag          !! Surface drag

    ! local declaration
    INTEGER(short)                                :: ji, jv
    REAL(stnd)                                    :: speed, zg, zdphi, ztvd, ztvs, zdu2
    REAL(stnd)                                    :: zri, cd_neut, zscf, cd_tmp

    ! initialisation

    ! test if we have to work with q_cdrag or to calcul it

    IF (.NOT.ldq_cdrag_from_gcm) THEN
        DO ji=1,kjpindex
          !
          ! 1. computes wind speed
          !
          speed = SQRT (u(ji)*u(ji) + v(ji)*v(ji))
          !
          ! 2. computes geopotentiel
          !
          zg = zlev(ji) * cte_grav
          zdphi = zg/cp_air
          !
          ! 3. virtual air temperature at the surface
          !
          ztvd = (temp_air(ji) + zdphi / (un + rvtmp2 * qair(ji))) * (un + retv * qair(ji)) 
          !
          ! 4. virtual surface temperature
          !
          ztvs = temp_sol(ji) * (un + retv * qsurf(ji))
          !
          ! 5. squared wind shear
          !
          zdu2 = MAX(cepdu2,speed**2)
          !
          ! 6. Richardson number
          !
          zri = zg * (ztvd - ztvs) / (zdu2 * ztvd)
          zri = MAX(MIN(zri,5.),-5.)
          !
          ! 7. Computing the drag coefficient
          !    We add the add the height of the vegetation to the level height to take into account
          !    that the level seen by the vegetation is actually the top of the vegetation. Then we 
          !    we can subtract the displacement height.
          !
          cd_neut = (ct_karman / LOG( (zlev(ji) + roughheight(ji)) / z0(ji) )) ** 2 
          !
          ! 7.1 Stable case
          !
          IF (zri .GE. zero) THEN
              zscf = SQRT(un + cd * ABS(zri))
              cd_tmp=cd_neut/(un + trois * cb * zri * zscf)
          ELSE
              !
              ! 7.2 Unstable case
              !
              zscf = un / (un + trois * cb * cc * cd_neut * SQRT(ABS(zri) * &
                   & ((zlev(ji) + roughheight(ji)) / z0(ji))))
              cd_tmp=cd_neut * (un - trois * cb * zri * zscf)
          ENDIF

          ! dont let it go to low else the surface uncouples
          q_cdrag(ji) = MAX(cd_tmp, 1.e-4/MAX(speed,min_wind))
!!
!!        In some situations it might be useful to give an upper limit on the cdrag as well. 
!!        The line here should then be uncommented.
!!          q_cdrag(ji) = MIN(q_cdrag(ji), 0.5/MAX(speed,min_wind))

        END DO

        IF (long_print) WRITE (numout,*) ' not ldqcdrag_from_gcm : diffuco_aero done '

    END IF

  END SUBROUTINE diffuco_aero

  !! This routine computes partial beta coefficient : snow sublimation 
  !!
  SUBROUTINE diffuco_snow (kjpindex, dtradia, qair, qsatt, rau, u, v, q_cdrag, &
       & snow, frac_nobio, totfrac_nobio, snow_nobio, vbeta1)

    ! interface description
    ! input scalar 
    INTEGER(long), INTENT(in)                               :: kjpindex   !! Domain size
    REAL(stnd), INTENT (in)                                 :: dtradia    !! Time step in seconds
    ! input fields
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: qair       !! Lowest level specific humidity
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: qsatt      !! Surface saturated humidity
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: rau        !! Density
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: u          !! Lowest level wind speed 
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: v          !! Lowest level wind speed
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: q_cdrag    !! Surface drag
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: snow       !! Snow mass
    REAL(stnd),DIMENSION (kjpindex,nnobio), INTENT (in)     :: frac_nobio !! Fraction of ice,lakes,cities,...
    REAL(stnd),DIMENSION (kjpindex,nnobio), INTENT (in)     :: snow_nobio !! Snow on ice,lakes,cities,...
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: totfrac_nobio!! Total fraction of ice+lakes+cities+...
    ! output fields
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)           :: vbeta1     !! Beta for sublimation

    ! local declaration
    REAL(stnd)                                              :: subtest, zrapp, speed, vbeta1_add
    INTEGER(short)                                          :: ji, jv

    !
    ! 1. beta coefficient for sublimation for snow on vegetation
    !
    DO ji=1,kjpindex
       ! Fraction of mesh that can sublimate snow
       vbeta1(ji) = (un - totfrac_nobio(ji)) * MAX( MIN(snow(ji)/snowcri,un), zero)
       !
       ! -- Limitation of sublimation in case of snow amounts smaller than
       !    the atmospheric demande. 
       !
       speed = MAX(min_wind, SQRT (u(ji)*u(ji) + v(ji)*v(ji)))
       !
       subtest = dtradia * vbeta1(ji) * speed * q_cdrag(ji) * rau(ji) * &
               & ( qsatt(ji) - qair(ji) )
       !  
       IF ( subtest .GT. zero ) THEN
          zrapp = snow(ji) / subtest
          IF ( zrapp .LT. un ) THEN
             vbeta1(ji) = vbeta1(ji) * zrapp
          ENDIF
       ENDIF
       !
    END DO

    !
    ! 2. add beta coefficient for other surface types.
    !
    DO jv = 1, nnobio
      !
      IF ( jv .EQ. iice ) THEN
        !
        !  Land ice is of course a particular case
        !
        DO ji=1,kjpindex
          vbeta1(ji) = vbeta1(ji) + frac_nobio(ji,jv)
        ENDDO
        !
      ELSE
        !
        DO ji=1,kjpindex
           !
           vbeta1_add = frac_nobio(ji,jv) * MAX( MIN(snow_nobio(ji,jv)/snowcri,un), zero)
           !
           ! -- Limitation of sublimation in case of snow amounts smaller than
           !    the atmospheric demande. 
           !
           speed = MAX(min_wind, SQRT (u(ji)*u(ji) + v(ji)*v(ji)))
           !
           subtest = dtradia * vbeta1_add * speed * q_cdrag(ji) * rau(ji) * &
                & ( qsatt(ji) - qair(ji) )
           !  
           IF ( subtest .GT. zero ) THEN
              zrapp = snow_nobio(ji,jv) / subtest
              IF ( zrapp .LT. un ) THEN
                 vbeta1_add = vbeta1_add * zrapp
              ENDIF
           ENDIF
           !
           vbeta1(ji) = vbeta1(ji) + vbeta1_add
           !
        ENDDO
        !
      ENDIF
      !
    ENDDO

    IF (long_print) WRITE (numout,*) ' diffuco_snow done '

  END SUBROUTINE diffuco_snow

  !! This routine computes partial beta coefficient : interception for each type of vegetation
  !!
  SUBROUTINE diffuco_inter (kjpindex, dtradia, qair, qsatt, rau, u, v, q_cdrag, veget, &
     & qsintveg, qsintmax, rstruct, vbeta2)

    ! interface description
    ! input scalar 
    INTEGER(long), INTENT(in)                               :: kjpindex   !! Domain size
    REAL(stnd), INTENT (in)                                 :: dtradia    !! Time step in seconds
    ! input fields
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: qair       !! Lowest level specific humidity
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: qsatt      !! Surface saturated humidity
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: rau        !! Density
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: u          !! Lowest level wind speed 
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: v          !! Lowest level wind speed
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: q_cdrag    !! Surface drag
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)        :: veget      !! vegetation fraction for each type
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)        :: qsintveg   !! Water on vegetation due to interception
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)        :: qsintmax   !! Maximum water on vegetation
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)        :: rstruct    !! STOMATE: architectural resistance
    ! output fields
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (out)       :: vbeta2     !! Beta for interception loss

    ! local declaration
    INTEGER(short)                                :: ji, jv
    REAL(stnd)                                    :: zqsvegrap, ziltest, zrapp, speed

!     print*,'TEST NON PRISE EN COMPTE DE RSTRUCT VBETA2'
    DO jv = 1,nvm

      !
      ! 1. beta coefficient for vegetation interception 
      !

      DO ji=1,kjpindex

        vbeta2(ji,jv) = zero

        IF (veget(ji,jv) .GT. min_sechiba .AND. qsintveg(ji,jv) .GT. zero ) THEN

            zqsvegrap = zero
            IF (qsintmax(ji,jv) .GT. min_sechiba ) THEN
                zqsvegrap = MAX(zero, qsintveg(ji,jv) / qsintmax(ji,jv))
            END IF
            !
            speed = MAX(min_wind, SQRT (u(ji)*u(ji) + v(ji)*v(ji)))
            !  -- Interception loss: IL
!           vbeta2(ji,jv) = veget(ji,jv) * zqsvegrap * (un / (un + speed * q_cdrag(ji) * rstruct(ji,jv)))
! TEST FH
            vbeta2(ji,jv) = veget(ji,jv) * zqsvegrap

            !
            !  -- Limitation of IL by the water stored on the leaf. 
            !     A first approximation of IL is obtained with the old values of
            !     qair and qsol_sat: function of temp-sol and pb. (see call of qsatcalc)
            !
            ziltest = dtradia * vbeta2(ji,jv) * speed * q_cdrag(ji) * rau(ji) * &
               & ( qsatt(ji) - qair(ji) )
            IF ( ziltest .GT. zero ) THEN
                zrapp = qsintveg(ji,jv) / ziltest
                IF ( zrapp .LT. un ) THEN
                    vbeta2(ji,jv) = vbeta2(ji,jv) * zrapp
                ENDIF
            ENDIF
        END IF

      END DO

    END DO

    IF (long_print) WRITE (numout,*) ' diffuco_inter done '

  END SUBROUTINE diffuco_inter

  !! This routine computes partial beta coefficient : bare soil
  !!
  SUBROUTINE diffuco_bare (kjpindex, dtradia, u, v, q_cdrag, hdry, evapot, humrel, veget, vbeta4, rsol)

    ! interface description
    ! input scalar 
    INTEGER(long), INTENT(in)                               :: kjpindex   !! Domain size
    REAL(stnd), INTENT (in)                                 :: dtradia    !! Time step in seconds
    ! input fields
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: u          !! Lowest level wind speed 
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: v          !! Lowest level wind speed
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: q_cdrag    !! Surface drag
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: hdry       !! Dry soil heigth in meters
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: evapot     !! Soil Potential Evaporation
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)        :: humrel     !! Soil moisture stress
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)        :: veget      !! Type of vegetation fraction
    ! output fields
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)           :: vbeta4     !! Beta for bare soil evpaoration
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)           :: rsol       !! resistance for bare soil evaporation

    ! local declaration
    INTEGER(short)                                :: ji, jv
    REAL(stnd)                                    :: speed

    DO ji = 1, kjpindex

      rsol(ji) = - un
      vbeta4(ji) = zero

      !     
      ! 1.   Soil resistance and beta for bare soil
      !      note: veget (  ,1) contains the fraction of bare soil
      !            see hydrol module
      !
      IF (veget(ji,1) .GE. min_sechiba) THEN
         !
         speed = MAX(min_wind, SQRT (u(ji)*u(ji) + v(ji)*v(ji)))
         !
! on modifie le rsol pour que la resistance croisse subitement si on s'approche
! du fond. En gros, rsol=hdry*rsol_cste pour hdry < 1m70

         rsol(ji) =  ( hdry(ji) + 1./(10.*(2.-hdry(ji))+1.e-10)**2 ) * rsol_cste

!MAF bug fix on SUN
!MAF         vbeta4(ji) = veget(ji,1) *humrel(ji,1)* (un / (un + q_cdrag(ji) * rsol(ji)))
         vbeta4(ji) = veget(ji,1) *humrel(ji,1)* (un / (un + speed * q_cdrag(ji) * rsol(ji)  ))
      END IF

    END DO

    IF (long_print) WRITE (numout,*) ' diffuco_bare done '

  END SUBROUTINE diffuco_bare

  !! This routine computes partial beta coefficient : transpiration for each type of vegetation
  !!
  SUBROUTINE diffuco_trans (kjpindex, dtradia, swnet, temp_air, pb, qair, rau, u, v, q_cdrag, humrel, &
                            veget, veget_max, lai, qsintveg, qsintmax, vbeta3, rveget, rstruct, cimean, vbetaco2)  

    ! interface description
    ! input scalar 
    INTEGER(long), INTENT(in)                               :: kjpindex         !! Domain size
    REAL(stnd), INTENT (in)                                 :: dtradia          !! Time step in seconds
    ! input fields
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: swnet            !! Short wave net flux in
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: temp_air         !! Air temperature in Kelvin
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: pb               !! Lowest level pressure
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: qair             !! Lowest level specific humidity
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: rau              !! Density
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: u                !! Lowest level wind speed 
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: v                !! Lowest level wind speed
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: q_cdrag          !! Surface drag
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)        :: humrel           !! Soil moisture stress
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)        :: veget            !! Type of vegetation fraction
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)        :: veget_max        !! Max. vegetation fraction (LAI -> infty)
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)        :: lai              !! Leaf area index
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)        :: qsintveg         !! Water on vegetation due to interception
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)        :: qsintmax         !! Maximum water on vegetation
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)        :: rstruct          !! STOMATE
    ! output fields
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (out)       :: vbeta3           !! Beta for Transpiration 
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (out)       :: rveget           !! Surface resistance of vegetation
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (out)       :: cimean           !! STOMATE
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (out)       :: vbetaco2         !! STOMATE

    ! local declaration
    INTEGER(short)                                :: ji, jv
    REAL(stnd)                                    :: speed
    REAL(stnd), DIMENSION(kjpindex)               :: zdefconc, zqsvegrap
    REAL(stnd), DIMENSION(kjpindex)               :: qsatt

    !
    ! 1.  Moisture concentration at the leaf level.
    !
    CALL qsatcalc (kjpindex, temp_air, pb, qsatt)
    zdefconc(:) = rau(:) * MAX( qsatt(:) - qair(:), zero )

    !
    ! 2. beta coefficient for vegetation transpiration
    !

!     print*,'TEST NON PRISE EN COMPTE DE RSTRUCT VBETA3'
    DO jv = 1,nvm

      rveget(:,jv) = - un
      vbeta3(:,jv) = zero

      zqsvegrap(:) = zero

      DO ji = 1, kjpindex

         speed = MAX(min_wind, SQRT (u(ji)*u(ji) + v(ji)*v(ji)))

         IF (qsintmax(ji,jv) .GT. min_sechiba) THEN
            zqsvegrap(ji) = MAX(zero, qsintveg(ji,jv) / qsintmax(ji,jv))
         ENDIF
         
         IF ( ( veget(ji,jv)*lai(ji,jv) .GT. min_sechiba  ) .AND. &
              ( kzero(jv) .GT. min_sechiba ) .AND. &
              ( swnet(ji) .GT. min_sechiba ) ) THEN
            
            rveget(ji,jv) = (( swnet(ji) + rayt_cste ) / swnet(ji) ) &
                 * ((defc_plus + (defc_mult * zdefconc(ji) )) / kzero(jv)) * (un / lai(ji,jv))
!           vbeta3(ji,jv) = veget(ji,jv) * (un - zqsvegrap(ji)) * humrel(ji,jv) * &
!                (un / (un + speed * q_cdrag(ji) * (rveget(ji,jv) + rstruct(ji,jv))))
! TEST FH
            vbeta3(ji,jv) = veget(ji,jv) * (un - zqsvegrap(ji)) * humrel(ji,jv) * &
                 (un / (un + speed * q_cdrag(ji) * rveg_cste * rveget(ji,jv)  ))

         ENDIF

      ENDDO

    ENDDO

    ! STOMATE

    DO jv = 1, nvm
      DO ji = 1, kjpindex
        cimean(ji,jv) = zero
        vbetaco2(ji,jv) = zero
      ENDDO
    ENDDO

    IF (long_print) WRITE (numout,*) ' diffuco_trans done '

  END SUBROUTINE diffuco_trans

  !! This routine computes partial beta coefficient : transpiration for each type of vegetation
  !! STOMATE: this routine now calculates also the assimilation using the Farqhuar & al (1980) formulation
  !!
  SUBROUTINE diffuco_trans_co2 (kjpindex, dtradia, swdown, temp_air, pb, qair, rau, u, v, q_cdrag, humrel, &
                                assim_param, ccanopy, &
                                veget, veget_max, lai, qsintveg, qsintmax, vbeta3, rveget, rstruct, cimean, vbetaco2)

    ! interface description
    ! input scalar 
    INTEGER(long), INTENT(in)                               :: kjpindex         !! Domain size
    REAL(stnd), INTENT (in)                                 :: dtradia          !! Time step in seconds
    ! input fields
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: swdown           !! Downwelling short wave flux
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: temp_air         !! Air temperature in Kelvin
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: pb               !! Lowest level pressure
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: qair             !! Lowest level specific humidity
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: rau              !! Density
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: u                !! Lowest level wind speed 
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: v                !! Lowest level wind speed
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: q_cdrag          !! Surface drag
    REAL(stnd),DIMENSION (kjpindex,nvm,npco2), INTENT (in)  :: assim_param      !! min+max+opt temps, vcmax, vjmax for photosynthesis
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: ccanopy          !! STOMATE: CO2 concentration inside the canopy
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)        :: humrel           !! Soil moisture stress
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)        :: veget            !! Type of vegetation fraction
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)        :: veget_max        !! Max. vegetation fraction (LAI -> infty)
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)        :: lai              !! Leaf area index
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)        :: qsintveg         !! Water on vegetation due to interception
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)        :: qsintmax         !! Maximum water on vegetation
    ! output fields
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (out)       :: vbeta3           !! Beta for Transpiration 
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (out)       :: rveget           !! Surface resistance of vegetation
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (out)       :: rstruct          !! STOMATE
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (out)       :: cimean           !! STOMATE
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (out)       :: vbetaco2         !! STOMATE

    ! local declaration
    REAL(stnd),DIMENSION (kjpindex,nvm)  :: vcmax
    REAL(stnd),DIMENSION (kjpindex,nvm)  :: vjmax
    REAL(stnd),DIMENSION (kjpindex,nvm)  :: tmin
    REAL(stnd),DIMENSION (kjpindex,nvm)  :: topt
    REAL(stnd),DIMENSION (kjpindex,nvm)  :: tmax
    INTEGER(short)                       :: ji, jv, jl
    REAL(stnd), DIMENSION(kjpindex)      :: leaf_ci_lowest 
    INTEGER(short), DIMENSION(kjpindex)  :: ilai
    REAL(stnd), DIMENSION(kjpindex)      :: zqsvegrap
    REAL(stnd)                           :: speed
    ! STOMATE:
    LOGICAL, DIMENSION(kjpindex)         :: assimilate, calculate
    INTEGER                              :: nic,inic
    INTEGER, DIMENSION(kjpindex)         :: index_calc
    INTEGER                              :: nia,inia,nina,inina
    INTEGER, DIMENSION(kjpindex)         :: index_assi,index_non_assi
    REAL(stnd), PARAMETER                :: laimax = 12.
    REAL(stnd), PARAMETER                :: xc4_1 = .83
    REAL(stnd), PARAMETER                :: xc4_2 = .93
    REAL(stnd), DIMENSION(kjpindex)      :: vc2, vj2
    REAL(stnd), DIMENSION(kjpindex)      :: assimi
    REAL(stnd)                           :: x_1,x_2,x_3,x_4,x_5,x_6
    REAL(stnd), DIMENSION(kjpindex)      :: gstop, gs
    REAL(stnd), DIMENSION(kjpindex)      :: Kc, Ko, CP
    REAL(stnd), DIMENSION(kjpindex)      :: vc, vj
    REAL(stnd), DIMENSION(kjpindex)      :: kt, rt
    REAL(stnd), DIMENSION(kjpindex)      :: air_relhum
    REAL(stnd), DIMENSION(kjpindex)      :: water_lim, temp_lim
    REAL(stnd), DIMENSION(kjpindex)      :: gstot
    REAL(stnd), DIMENSION(kjpindex)      :: assimtot
    REAL(stnd), DIMENSION(kjpindex)      :: leaf_gs_top   !! stomatal conductance at topmost level
    REAL(stnd), DIMENSION(nlai+1)        :: laitab        !! tabulated LAI steps
    REAL(stnd), DIMENSION(kjpindex)      :: qsatt
    REAL(stnd), DIMENSION(nvm,nlai)      :: light         !! fraction of light that gets through

    !
    ! calculate LAI steps
    !
    DO jl = 1, nlai+1
      laitab(jl) = laimax*(EXP(.15*REAL(jl-1,stnd))-1.)/(EXP(.15*REAL(nlai,stnd))-1.)
    ENDDO
    !
    ! calculate light fraction that comes through at a given LAI for each vegetation type
    !
    DO jl = 1, nlai
      !
      DO jv = 1, nvm
        !
        light(jv,jl) = exp( -ext_coef(jv)*laitab(jl) )
        !
      ENDDO
      !
    ENDDO 
    !
    ! 1. Photosynthesis parameters
    !
    !
    ! temperatures in K
    !
    tmin(:,:) = assim_param(:,:,itmin)
    tmax(:,:) = assim_param(:,:,itmax)
    topt(:,:) = assim_param(:,:,itopt)
    !
    vcmax(:,:) = assim_param(:,:,ivcmax)
    vjmax(:,:) = assim_param(:,:,ivjmax)
    !
    ! estimation of relative humidity of the air
    !
    CALL qsatcalc (kjpindex, temp_air, pb, qsatt)
    air_relhum(:) = &
      ( qair(:) * pb(:) / (0.622+qair(:)*0.378) ) / &
      ( qsatt(:)*pb(:) / (0.622+qsatt(:)*0.378 ) )
    !
    DO jv = 1,nvm
      !
      ! 2. beta coefficient for vegetation transpiration
      !
      rveget(:,jv) = - un
      !
      vbeta3(:,jv) = zero
      vbetaco2(:,jv) = zero
      !
      cimean(:,jv) = ccanopy(:)
      !
      ! mask that contains points where there is photosynthesis
      !
      nia=0
      nina=0
      !
      DO ji=1,kjpindex
        !
      	IF ( ( lai(ji,jv) .GT. 0.01 ) .AND. &
              ( veget(ji,jv) .GT. 1.E-8 ) .AND. &
              ( swdown(ji) .GT. min_sechiba ) .AND. &
              ( temp_air(ji) .GT. tmin(ji,jv) ) .AND. &
              ( temp_air(ji) .LT. tmax(ji,jv) ) .AND. &
              ( humrel(ji,jv) .GT. min_sechiba )       ) then
          !
          assimilate(ji) = .TRUE.
          nia=nia+1
          index_assi(nia)=ji
          !
        ELSE
          !
          assimilate(ji) = .FALSE.
          nina=nina+1
          index_non_assi(nina)=ji
          !
        ENDIF
        !
      ENDDO
      !
      gstot(:) = zero
      assimtot(:) = zero
      !
      zqsvegrap(:) = zero
      WHERE (qsintmax(:,jv) .GT. min_sechiba)
          zqsvegrap(:) = MAX(zero, qsintveg(:,jv) / qsintmax(:,jv))
      ENDWHERE
      !
      WHERE ( assimilate(:) )
        water_lim(:) = MIN( 2.*humrel(:,jv), 1. )
      ENDWHERE
      ! give a default value of ci for all pixel that do not assimilate
      DO jl=1,nlai
         DO inina=1,nina
            leaf_ci(index_non_assi(inina),jv,jl) = ccanopy(index_non_assi(inina)) * .667_stnd
         ENDDO
      ENDDO
      !
      ilai(:) = 1
      !
      ! Here is calculated photosynthesis (Farqhuar et al. 80)
      ! and stomatal conductance (Ball & al. 86)
      !
      ! Calculating temperature dependent parameters
      !
      IF ( is_c4(jv) )  THEN
        !
        ! Case of C4 plants
        !
        IF (nia .GT. 0) then
!OCL NOVREC
          DO inia=1,nia
            !
            x_1 = 0.177 * EXP( 0.069*(temp_air(index_assi(inia))-tp_00) )  
            ! = 2.0**(((temp_air(index_assi(inia))-tp_00)-25.0)/10.0)
            !
            kt(index_assi(inia)) = 0.7 * x_1 * 1.e6
            rt(index_assi(inia)) = 0.8 * x_1 / &
              ( 1.0 + EXP(1.3*(temp_air(index_assi(inia))-tmax(index_assi(inia),jv))) )
            !
            vc(index_assi(inia)) = vcmax(index_assi(inia),jv) & 
                * 0.39 * x_1 * water_lim(index_assi(inia)) / &
!                 * 0.39 * x_1  / &
                ( (1.0+EXP(0.3*(tmin(index_assi(inia),jv)-temp_air(index_assi(inia))))) &
                * (1.0+EXP(0.3*(temp_air(index_assi(inia))-topt(index_assi(inia),jv)))) )
             !
          ENDDO
        ENDIF
        !
        IF (nina .GT. 0) then
          !
!OCL NOVREC
          DO inina=1,nina
            !
            kt(index_non_assi(inina)) = 0.0
            rt(index_non_assi(inina)) = 0.0
            vc(index_non_assi(inina)) = 0.0
            !
          ENDDO
          !
        ENDIF
        !
      ELSE
        !
        ! Case of C3 plants
        !
        IF (nia .GT. 0) then
          !
!OCL NOVREC
          DO inia=1,nia
            !
            temp_lim(index_assi(inia)) = &
              (temp_air(index_assi(inia))-tmin(index_assi(inia),jv)) * &
              (temp_air(index_assi(inia))-tmax(index_assi(inia),jv))
            temp_lim(index_assi(inia)) = temp_lim(index_assi(inia)) / &
              (temp_lim(index_assi(inia))-(temp_air(index_assi(inia))-&
               topt(index_assi(inia),jv))**2.)
            !
            Kc(index_assi(inia)) = 39.09 * EXP(.085*(temp_air(index_assi(inia))-tp_00))
            !
            Ko(index_assi(inia)) = 2.412 * 210000. &
                * EXP(.085*(temp_air(index_assi(inia))-tmin(index_assi(inia),jv))) / &
                  (temp_air(index_assi(inia))-tmin(index_assi(inia),jv))
            !
            CP(index_assi(inia)) = 42. * EXP( 9.46*(temp_air(index_assi(inia))-tp_00-25.)/&
                                                           temp_air(index_assi(inia)) )
            !
            vc(index_assi(inia)) = vcmax(index_assi(inia),jv) * &
               temp_lim(index_assi(inia)) * water_lim(index_assi(inia))
!               temp_lim(index_assi(inia)) 
            vj(index_assi(inia)) = vjmax(index_assi(inia),jv) * &
               temp_lim(index_assi(inia)) * water_lim(index_assi(inia))
!                temp_lim(index_assi(inia))
            !
          ENDDO
          !
        ENDIF
        !
        IF (nina .GT. 0) then
          !
!OCL NOVREC
          DO inina=1,nina
            ! 
            temp_lim(index_non_assi(inina)) = 0.0
            Kc(index_non_assi(inina)) = 0.0
            Ko(index_non_assi(inina)) = 0.0
            CP(index_non_assi(inina)) = 0.0
            !
            vc(index_non_assi(inina)) = 0.0
            vj(index_non_assi(inina)) = 0.0
            !
          ENDDO
          !
        ENDIF
        !
      ENDIF      ! C3/C4
      !
      ! estimate assimilation and conductance for each LAI level
      !
      DO jl = 1, nlai
        !
        nic=0
        !
        calculate(:) = .FALSE.
        !
        IF (nia .GT. 0) then
          !
!OCL NOVREC
          DO inia=1,nia
            !
            calculate(index_assi(inia)) = (laitab(jl) .LE. lai(index_assi(inia),jv) )
            !
            IF ( calculate(index_assi(inia)) ) THEN
              !
              nic=nic+1
              index_calc(nic)=index_assi(inia)
              !
            ENDIF
            !
          ENDDO
          !
        ENDIF
        !
        ! Vmax is scaled into the canopy due to reduction of nitrogen
        !
        x_1 = ( un - .7_stnd * ( un - light(jv,jl) ) )
        !
        IF ( nic .GT. 0 ) THEN
          !
          DO inic=1,nic
            !
            vc2(index_calc(inic)) = vc(index_calc(inic)) * x_1
            vj2(index_calc(inic)) = vj(index_calc(inic)) * x_1
            !
          ENDDO
          !
        ENDIF
        !
        IF ( is_c4(jv) )  THEN
          !
          ! assimilation for C4 plants (Collatz & al. 91)
          !
          DO ji = 1, kjpindex
            !
            assimi(ji) = 0.
            !
          ENDDO
          !
          IF (nic .GT. 0) THEN
            !
!OCL NOVREC
            DO inic=1,nic
              !
              x_1 = - ( vc2(index_calc(inic)) + 0.092 * 2.3* swdown(index_calc(inic)) * &
                    ext_coef(jv) * light(jv,jl) )
              x_2 = vc2(index_calc(inic)) * 0.092 * 2.3 * swdown(index_calc(inic)) * &
                    ext_coef(jv) * light(jv,jl) 
              x_3 = ( -x_1 - sqrt( x_1*x_1 - 4.0 * xc4_1 * x_2 ) ) / (2.0*xc4_1)
              x_4 = - ( x_3 + kt(index_calc(inic)) * leaf_ci(index_calc(inic),jv,jl) * &
                    1.0e-6 )
              x_5 = x_3 * kt(index_calc(inic)) * leaf_ci(index_calc(inic),jv,jl) * 1.0e-6
              assimi(index_calc(inic)) = ( -x_4 - sqrt( x_4*x_4 - 4. * xc4_2 * x_5 ) ) / (2.*xc4_2)
              assimi(index_calc(inic)) = assimi(index_calc(inic)) - &
                                                      rt(index_calc(inic))
              !
            ENDDO
            !
          ENDIF
          !
        ELSE
          !
          ! assimilation for C3 plants (Farqhuar & al. 80)
          !
          DO ji = 1, kjpindex
            !
            assimi(ji) = 0.
            !
          ENDDO
          !
          IF (nic .GT. 0) THEN
            !
!OCL NOVREC
            DO inic=1,nic
              !
              x_1 = vc2(index_calc(inic)) * leaf_ci(index_calc(inic),jv,jl) / &
                    ( leaf_ci(index_calc(inic),jv,jl) + Kc(index_calc(inic)) * &
                    ( 1._stnd + 210000._stnd / Ko(index_calc(inic)) ) )
              x_2 = .8855_stnd*swdown(index_calc(inic))*ext_coef(jv)*light(jv,jl)
              x_3 = x_2+vj2(index_calc(inic))
              x_4 = ( x_3 - sqrt( x_3*x_3 - (4._stnd*.7_stnd*x_2*vj2(index_calc(inic))) ) ) / &
                    (2._stnd*.7_stnd)
              x_5 = x_4 * leaf_ci(index_calc(inic),jv,jl) / &
                    ( 4.5_stnd *  leaf_ci(index_calc(inic),jv,jl) + &
                    10.5_stnd*CP(index_calc(inic)) ) 
              x_6 = MIN( x_1, x_5 )
              assimi(index_calc(inic)) = x_6 * ( 1._stnd - CP(index_calc(inic))/&
                 leaf_ci(index_calc(inic),jv,jl) ) - .011_stnd * vc2(index_calc(inic))
              !
            ENDDO
            !
          ENDIF
          !
        ENDIF
        !
        IF (nic .GT. 0) THEN
          !
!OCL NOVREC
          DO inic=1,nic
            !
            ! estimate conductance (Ball & al. 86)
            !
!            gs(index_calc(inic)) = water_lim(index_calc(inic)) * &
             gs(index_calc(inic)) = &
                  ( gsslope(jv) * assimi(index_calc(inic)) * &
                  air_relhum(index_calc(inic)) / ccanopy(index_calc(inic)) ) &
                  + gsoffset(jv)
            gs(index_calc(inic)) = MAX( gs(index_calc(inic)), gsoffset(jv) )
            !
            ! the new ci is calculated with
            ! dci/dt=(ccanopy-ci)*gs/1.6-A
            ! ci=ci+((ccanopy(index_calc(inic))-ci)*gs/1.6-&
            !    assimi(index_calc(inic)))*dtradia
            ! we verify that ci is not out of possible values
            !
            !to avoid some problem of numerical stability, the leaf_ci is bufferized
            leaf_ci(index_calc(inic),jv,jl) =(leaf_ci(index_calc(inic),jv,jl)*5.+ &
                MIN( ccanopy(index_calc(inic)), MAX( CP(index_calc(inic)), &
                   ( ccanopy(index_calc(inic)) - 1.6_stnd * assimi(index_calc(inic)) / &
                     gs(index_calc(inic)) ) ) ))/6.

            !
            ! this might be the last level for which Ci is calculated. Store it for 
            ! initialization of the remaining levels of the Ci array.
            !
            leaf_ci_lowest(index_calc(inic)) = leaf_ci(index_calc(inic),jv,jl)
            !
            ! total assimilation and conductance
            assimtot(index_calc(inic)) = assimtot(index_calc(inic)) + &
              assimi(index_calc(inic)) * (laitab(jl+1)-laitab(jl))
            gstot(index_calc(inic)) = gstot(index_calc(inic)) + &
              gs(index_calc(inic)) * (laitab(jl+1)-laitab(jl))
            !
            ilai(index_calc(inic)) = jl
            !
          ENDDO
          !
        ENDIF
        !
        ! keep stomatal conductance of topmost level
        !
        IF ( jl .EQ. 1 ) THEN
          !
          leaf_gs_top(:) = 0.
          !
          IF ( nic .GT. 0 ) then
            !
!OCL NOVREC
            DO inic=1,nic
              !
              leaf_gs_top(index_calc(inic)) = gs(index_calc(inic))
              !
            ENDDO
            !
          ENDIF
          !
        ENDIF
        !
        IF (nia .GT. 0) THEN
          !
!OCL NOVREC
          DO inia=1,nia
            !
            IF ( .NOT. calculate(index_assi(inia)) ) THEN
              !
              !   a) for plants that are doing photosynthesis, but whose LAI is lower
              !      than the present LAI step, initialize it to the Ci of the lowest
              !      canopy level
              !
              leaf_ci(index_assi(inia),jv,jl) = leaf_ci_lowest(index_assi(inia))
              !
            ENDIF
            !
          ENDDO
          !
        ENDIF
        !
      ENDDO  ! loop over LAI steps
      !
      ! final calculations: resistances
      !
      IF (nia .GT. 0) THEN
        !
!OCL NOVREC
        DO inia=1,nia
          !
          speed = MAX(min_wind, SQRT (u(index_assi(inia))*u(index_assi(inia)) + &
                   v(index_assi(inia))*v(index_assi(inia))))
          !
          ! conversion from mmol/m2/s to m/s
          !
          gstot(index_assi(inia)) = .0244*(temp_air(index_assi(inia))/tp_00)*&
             (1013./pb(index_assi(inia)))*gstot(index_assi(inia))
          gstop(index_assi(inia)) = .0244 * (temp_air(index_assi(inia))/tp_00)*&
             (1013./pb(index_assi(inia)))*leaf_gs_top(index_assi(inia))*&
              laitab(ilai(index_assi(inia))+1)
          !
          rveget(index_assi(inia),jv) = 1./gstop(index_assi(inia))
          !
          ! rstruct is the difference between rtot (=1./gstot) and rveget
          !
          rstruct(index_assi(inia),jv) = 1./gstot(index_assi(inia)) - &
               rveget(index_assi(inia),jv)
          !
          ! beta for transpiration
          !
!         vbeta3(index_assi(inia),jv) = veget_max(index_assi(inia),jv) * &
!           (un - zqsvegrap(index_assi(inia))) * &
!           (un / (un + speed * q_cdrag(index_assi(inia)) * (rveget(index_assi(inia),jv) + &
!            rstruct(index_assi(inia),jv))))
! TEST FH : pas de prise en compte de rstruct.
!         print*,'TEST BETA3 SANS RSTRUCT '
          vbeta3(index_assi(inia),jv) = veget_max(index_assi(inia),jv) * &
            (un - zqsvegrap(index_assi(inia))) * &
            (un / (un + speed * q_cdrag(index_assi(inia)) * rveg_cste * rveget(index_assi(inia),jv) ))
          !
          ! beta for assimilation. The difference is that surface 
          ! covered by rain (un - zqsvegrap(index_assi(inia))) is not taken into account
          ! 1.6 is conversion for H2O to CO2 conductance
          ! vbetaco2(index_assi(inia),jv) = veget_max(index_assi(inia),jv) * &
          !   (un / (un + q_cdrag(index_assi(inia)) * &
          !   (rveget(index_assi(inia),jv))))/1.6
          !
          vbetaco2(index_assi(inia),jv) = veget_max(index_assi(inia),jv) * &
             (un / (un + speed * q_cdrag(index_assi(inia)) * &
             (rveget(index_assi(inia),jv) + rstruct(index_assi(inia),jv)))) / 1.6
          !
          ! cimean is the "mean ci" calculated in such a way that assimilation 
          ! calculated in enerbil is equivalent to assimtot
          !
          cimean(index_assi(inia),jv) = ccanopy(index_assi(inia)) - &
             assimtot(index_assi(inia)) / &
             ( vbetaco2(index_assi(inia),jv)/veget_max(index_assi(inia),jv) * &
             rau(index_assi(inia)) * speed * q_cdrag(index_assi(inia)))
          !
        ENDDO
        !
      ENDIF
      !
    END DO         ! loop over vegetation types
    !
    IF (long_print) WRITE (numout,*) ' diffuco_trans_co2 done '

  END SUBROUTINE diffuco_trans_co2

  !! This routine combines previous partial beta coeeficient and calculates
  !! alpha and complete beta coefficient
  !!
  SUBROUTINE diffuco_comb (kjpindex, dtradia, humrel, rau, u, v, q_cdrag, pb, qair, temp_sol, temp_air, &
       & snow, veget, vbeta1, vbeta2, vbeta3 , vbeta4, valpha, vbeta)    

    ! interface description
    ! input scalar 
    INTEGER(long), INTENT(in)                               :: kjpindex         !! Domain size
    REAL(stnd), INTENT (in)                                 :: dtradia          !! Time step in seconds
    ! input fields
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: rau              !! Density
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: u                !! Lowest level wind speed 
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: v                !! Lowest level wind speed
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: q_cdrag          !! Surface drag
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: pb               !! Lowest level pressure
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: qair             !! Lowest level specific humidity
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: temp_sol         !! Skin temperature in Kelvin
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: temp_air         !! lower air temperature in Kelvin
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: snow             !! Snow mass
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)        :: veget            !! Fraction of vegetation type
    ! modified fields 
    REAL(stnd),DIMENSION (kjpindex), INTENT (inout)         :: vbeta1           !! Beta for sublimation
    REAL(stnd),DIMENSION (kjpindex), INTENT (inout)         :: vbeta4           !! Beta for Bare soil evaporation
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (inout)         :: humrel           !! Soil moisture stress
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (inout)     :: vbeta2           !! Beta for Interception for
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (inout)     :: vbeta3           !! Beta for Transpiration 
    ! output fields
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)           :: valpha           !! TotalAlpha coefficient
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)           :: vbeta            !! Total beta coefficient

    ! local declaration
    INTEGER(short)                                :: ji, jv
    REAL(stnd)                                    :: zevtest, zsoil_moist, zrapp
    REAL(stnd), DIMENSION(kjpindex)               :: vbeta2sum, vbeta3sum
    REAL(stnd), DIMENSION(kjpindex)               :: vegetsum, vegetsum2
    REAL(stnd), DIMENSION(kjpindex)               :: qsatt
    LOGICAL, DIMENSION(kjpindex)                  :: toveg, tosnow

    vbeta2sum(:) = 0.
    vbeta3sum(:) = 0.
    DO jv = 1, nvm
      vbeta2sum(:) = vbeta2sum(:) + vbeta2(:,jv)
      vbeta3sum(:) = vbeta3sum(:) + vbeta3(:,jv)
    ENDDO 

    !
    ! 1. The beta and alpha coefficients are calculated.
    !

    vbeta(:) = un
    valpha(:) = un

    !
    ! 2. if snow is lower than critical value
    !

    DO ji = 1, kjpindex

      IF  (snow(ji) .LT. snowcri) THEN

          vbeta(ji) = vbeta4(ji) + vbeta2sum(ji) + vbeta3sum(ji)

          IF (vbeta(ji) .LT. min_sechiba) THEN
             vbeta(ji) = zero
          END IF

      END IF

    ENDDO

    !
    ! 3. If we are in presence of dew.
    !

    ! for vectorization: some arrays
    vegetsum(:) = 0.
    DO jv = 1, nvm
      vegetsum(:) = vegetsum(:) + veget(:,jv)
    ENDDO
    vegetsum2(:) = 0.
    DO jv = 2, nvm
      vegetsum2(:) = vegetsum2(:) + veget(:,jv)
    ENDDO

    CALL qsatcalc (kjpindex, temp_sol, pb, qsatt)

    !
    ! 3.1 decide where the water goes (soil, vegetation, or snow)
    !     when air moisture exceeds saturation.
    ! 
    toveg(:) = .FALSE.
    tosnow(:) = .FALSE.
    DO ji = 1, kjpindex
      IF ( qsatt(ji) .LT. qair(ji) ) THEN
          IF (temp_air(ji) .GT. tp_00) THEN
              !
              ! 3.1.1  If it is not freezing dew is put into the 
              !        interception reservoir and on the bare soil.
              toveg(ji) = .TRUE.
          ELSE
              !
              ! 3.1.2  If it is freezing water is put into the 
              !        snow reservoir.
              tosnow(ji) = .TRUE.
          ENDIF
      ENDIF
    END DO

    ! 3.1.3 now modify valpha and vbetas where necessary.
    !
    ! 3.1.3.1 Soil and snow (2d)
    !
    DO ji = 1, kjpindex
      IF ( toveg(ji) ) THEN
        vbeta1(ji) = zero
        vbeta4(ji) = veget(ji,1)
        vbeta(ji) = vegetsum(ji)
        valpha(ji) = un
      ENDIF
      IF ( tosnow(ji) ) THEN
        vbeta1(ji) = un
        vbeta4(ji) = zero
        vbeta(ji) = un
        valpha(ji) = un
      ENDIF
    ENDDO
    !
    ! 3.1.3.2 vegetation (3d)
    !
    DO jv = 1, nvm
      !
      DO ji = 1, kjpindex
        !
        IF ( toveg(ji) ) then
           if (qsintcst.gt.min_sechiba) then
             vbeta2(ji,jv) = veget(ji,jv)
           else
             vbeta2(ji,jv) = 0.
           endif
        ENDIF
        IF ( tosnow(ji) ) vbeta2(ji,jv) = zero
        !
      ENDDO
      !
    ENDDO

    !
    ! 3.2  In any case there is no transpiration when air moisture is too high.
    !
    
    DO jv = 1, nvm
      DO ji = 1, kjpindex
        IF ( qsatt(ji) .LT. qair(ji) ) THEN
          vbeta3(ji,jv) = zero
          humrel(ji,jv) = zero
        ENDIF
      ENDDO
    ENDDO

    !
    ! 3.2_bis  In any case there is no interception loss on bare soil.
    !

    DO ji = 1, kjpindex
       IF ( qsatt(ji) .LT. qair(ji) ) THEN
          vbeta2(ji,1) = zero
       ENDIF
    ENDDO

    IF (long_print) WRITE (numout,*) ' diffuco_comb done '

  END SUBROUTINE diffuco_comb

  SUBROUTINE diffuco_raerod (kjpindex, u, v, q_cdrag, raero)
    !
    ! Simply computes the aerodynamic resistance. For the moment it is
    ! only used as a diagnostic but that may change !
    !
    IMPLICIT NONE
    !
    INTEGER(long), INTENT(in)                               :: kjpindex         !! Domain size
    ! input fields
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: u                !! Lowest level wind speed 
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: v                !! Lowest level wind speed
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)            :: q_cdrag          !! Surface drag
    ! output filed 
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)           :: raero            !! Aerodynamic resistance
    !
    ! local declaration
    INTEGER(short)                                :: ji
    REAL(stnd)                                    :: speed
    !
    DO ji=1,kjpindex
       !
       speed = MAX(min_wind, SQRT (u(ji)*u(ji) + v(ji)*v(ji)))
       raero(ji) = un / (q_cdrag(ji)*speed)
       !
    ENDDO
    !
  END SUBROUTINE diffuco_raerod

END MODULE diffuco




